/*
 *	@(#)MoverBehavior.java 1.1 98/04/02 18:27:44
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */

import java.awt.event.*;
import java.awt.AWTEvent;
import javax.media.j3d.*;
import java.util.Enumeration;
import javax.vecmath.*;

// Mover behavior class - used to allow viewer to move using arrow keys
class KeyBehavior extends Behavior
{
    WakeupOnAWTEvent w1 = new WakeupOnAWTEvent(KeyEvent.KEY_PRESSED);
    WakeupCriterion[] w2 = {w1};
    WakeupCondition w = new WakeupOr(w2);
    TransformGroup viewTransformGroup;
        
    public void initialize() 
	{
	 // Establish initial wakeup criteria
	 wakeupOn(w);
    }


    /**
     *  Override Behavior's stimulus method to handle the event.
     */
    public void processStimulus(Enumeration criteria) 
	{
	 WakeupOnAWTEvent ev;
	 WakeupCriterion genericEvt;
	 AWTEvent[] events;
   
   	 while (criteria.hasMoreElements()) 
	 {
	    genericEvt = (WakeupCriterion) criteria.nextElement();
	    if (genericEvt instanceof WakeupOnAWTEvent) 
		{
		 ev = (WakeupOnAWTEvent) genericEvt;
		 events = ev.getAWTEvent();
		 processManualEvent(events);
	    }
	 }
	 // Set wakeup criteria for next time
	 wakeupOn(w);
    }

    
    /**
     *  Process a keyboard event to move or rotate the viewer.
     */
    void processManualEvent(AWTEvent[] events) 
	{

	 for (int i = 0; i < events.length; ++i) 
	 {
	    if (events[i] instanceof KeyEvent) 
		{
		 KeyEvent event = (KeyEvent)events[i];
		 if (event.getKeyCode() == KeyEvent.VK_EQUALS) { continue; }
		 Transform3D t = new Transform3D();
		 viewTransformGroup.getTransform(t);
		 Vector3f viewDir = new Vector3f(0.5f, 0f, 0.5f);
		 Vector3f translation = new Vector3f();
		 t.get(translation);
		 t.transform(viewDir);
		 if (event.getKeyCode() == KeyEvent.VK_UP) { translation.z -= viewDir.z; }
		 else if (event.getKeyCode() == KeyEvent.VK_DOWN) { translation.z += viewDir.z; }
		 else if (event.getKeyCode() == KeyEvent.VK_RIGHT) { translation.x += viewDir.x; }
		 else if (event.getKeyCode() == KeyEvent.VK_LEFT) { translation.x -= viewDir.x; }
		 t.setTranslation(translation);
		 viewTransformGroup.setTransform(t);
	    }
	 }
    }

  
    /**
     *  Constructor 
     */
    public KeyBehavior(TransformGroup trans) 
	{
	 viewTransformGroup = trans;
	 Bounds bound = new BoundingSphere(new Point3d(0.0,0.0,0.0),10000.0);
	 this.setSchedulingBounds(bound);
    }
}



